# Generated by Django 2.2.2 on 2019-06-15 08:37


from django.db import migrations, models
import django.utils.timezone
import isc_common.fields.code_field


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='History',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateTimeField(db_index=True, default=django.utils.timezone.now)),
                ('method', isc_common.fields.code_field.CodeField()),
                ('path', isc_common.fields.code_field.CodeField()),
                ('data', django.db.models.JSONField(default=dict)),
            ],
            options={
                'verbose_name': 'History',
            },
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-18 12:45

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('events', '0002_auto_20191117_0959'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event_types',
            name='props',
            field=bitfield.models.BitField((('isEvent', 'Является событием'), ('compulsory_reading', 'Обязательное прочтение')), db_index=True, default=0),
        ),
    ]



import logging

from django.db.models import Model, BooleanField, BigAutoField, DateTimeField, Transform, CharField, TextField
from django.utils import timezone

from isc_common import setAttr
from isc_common.managers.common_manager import CommonManager, CommonQuerySet

logger = logging.getLogger(__name__)


class AuditQuerySet(CommonQuerySet):
    def _check_ed_izm_qty(self, **kwargs):
        if kwargs.get('ed_izm') != None and kwargs.get('qty') == None or kwargs.get('ed_izm') == None and kwargs.get('qty') != None:
            raise Exception(f'ed_izm and qty not be along')

    def __init__(self, model=None, query=None, using=None, hints=None, alive_only=True):
        self.alive_only = alive_only
        super().__init__(model=model, query=query, using=using, hints=hints, alive_only=alive_only)

    def rearrange_parent(self, json):
        _criteria = json.get('data').get('criteria')
        if isinstance(_criteria, list):
            for criterion in _criteria:
                if criterion.get('fieldName') == 'parent_id':
                    item_id = criterion.get('value')
                    if (item_id, int):
                        criteria_lst = [criterion for criterion in _criteria if criterion.get('fieldName') == 'parent_id' and criterion.get('value') != None]
                        if len(criteria_lst) > 0:
                            setAttr(json.get('data'), 'criteria', criteria_lst)
        return json

    def soft_delete(self):
        res = super().update(deleted_at=timezone.now())
        if res > 0:
            return self
        else:
            return None

    def soft_restore(self):
        res = super().update(deleted_at=None)
        return res

    def hard_delete(self):
        res = super().delete()
        return res

    def alive(self):
        return self.filter(deleted_at=None)

    def dead(self):
        return self.exclude(deleted_at=None)

    def update(self, **kwargs):
        if kwargs.get('lastmodified') == None:
            setAttr(kwargs, 'lastmodified', timezone.now())
        return super().update(**kwargs)


class AuditManager(CommonManager):
    def __init__(self, *args, **kwargs):
        self.alive_only = kwargs.pop('alive_only', True)
        super().__init__(*args, **kwargs)

    def get_queryset(self):
        if self.alive_only:
            return AuditQuerySet(model=self.model, alive_only=self.alive_only).filter(deleted_at=None)
        return AuditQuerySet(model=self.model, alive_only=self.alive_only)

    def hard_delete(self):
        return self.get_queryset().hard_delete()

    def soft_delete(self):
        return self.get_queryset().soft_delete()

    def soft_restore(self):
        return self.get_queryset().soft_restore()


class Manager(CommonManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_queryset(self):
        return AuditQuerySet(model=self.model)


class AuditModel(Model):
    id = BigAutoField(primary_key=True, verbose_name="Идентификатор")
    deleted_at = DateTimeField(verbose_name="Дата мягкого удаления", null=True, blank=True, db_index=True)
    editing = BooleanField(verbose_name="Возможность редактирования", default=True)
    deliting = BooleanField(verbose_name="Возможность удаления", default=True)
    lastmodified = DateTimeField(verbose_name='Последнее обновление', editable=False, db_index=True, default=timezone.now)

    @staticmethod
    def uncapitalize(str):
        from isc_common.common.functions import uncapitalize
        return uncapitalize(str)

    @staticmethod
    def dbl_qutes_str(str):
        from isc_common.common.functions import dbl_qutes_str
        return dbl_qutes_str(str)

    @staticmethod
    def qutes_str(str):
        from isc_common.common.functions import qutes_str
        return qutes_str(str)

    @property
    def idHidden(self):
        return not self == None

    @property
    def is_deleted(self):
        return self.deleted_at != None

    class Meta:
        abstract = True

    def soft_delete(self):
        self.deleted_at = timezone.now()
        self.save()

    def soft_restore(self):
        self.deleted_at = None
        self.save()

    objects = AuditManager()
    all_objects = AuditManager(alive_only=False)


class Dbl_spacesValue(Transform):
    lookup_name = 'delete_dbl_spaces'
    function = 'delete_dbl_spaces'


CharField.register_lookup(Dbl_spacesValue)
TextField.register_lookup(Dbl_spacesValue)


class Trim_Value(Transform):
    lookup_name = 'trim'
    function = 'trim'


CharField.register_lookup(Trim_Value)
TextField.register_lookup(Trim_Value)



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('tracker', '0022_auto_20190517_1802'),
    ]
    atomic = True

    operations = [
        # migrations.RunPython(fill),
    ]



# Generated by Django 2.1.5 on 2019-01-20 10:24

import django.contrib.auth.validators
import django.utils.timezone
from django.db import migrations, models

import isc_common.auth.managers.user_manager
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Признак удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('password', isc_common.fields.code_field.CodeField(verbose_name='пароль')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='последний вход')),
                ('username', isc_common.fields.code_field.CodeField(error_messages={'unique': 'Такой пользователь уже существует.'}, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='логин')),
                ('first_name', isc_common.fields.name_field.NameField(verbose_name='имя')),
                ('last_name', isc_common.fields.name_field.NameField(verbose_name='фамилия')),
                ('email', models.EmailField(blank=True, max_length=254, null=True, verbose_name='E-mail')),
                ('middle_name', isc_common.fields.name_field.NameField(verbose_name='отчетво')),
                ('permissions', django.db.models.JSONField(default=dict, null=True, blank=True,)),
            ],
            options={
                'verbose_name': 'Пользователь',
                'swappable': 'AUTH_USER_MODEL',
            },
            managers=[
                ('objects', isc_common.auth.managers.user_manager.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='UserGroup',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Признак удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('name', isc_common.fields.name_field.NameField()),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('code', isc_common.fields.code_field.CodeField(unique=True)),
                ('permissions', django.db.models.JSONField(default=dict, null=True, blank=True,)),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='isc_common.UserGroup')),
            ],
            options={
                'verbose_name': 'группа',
                'verbose_name_plural': 'группы',
            },
        ),
        migrations.CreateModel(
            name='Widgets_trees',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Признак удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('id_widget', isc_common.fields.code_field.CodeField()),
                ('name', isc_common.fields.name_field.NameField()),
                ('class_name', isc_common.fields.name_field.NameField()),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('structure', django.db.models.JSONField(default=dict)),
            ],
            options={
                'verbose_name': 'Таблица сохранения деревьев виджетов объектов системы',
            },
        ),
        migrations.AddField(
            model_name='user',
            name='usergroup',
            field=models.ManyToManyField(to='isc_common.UserGroup', verbose_name='группы'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-09 16:48

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0041_progresses_showcancel'),
    ]

    operations = [
        migrations.AddField(
            model_name='progresses',
            name='type',
            field=isc_common.fields.code_field.CodeStrictField(),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-16 10:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('history', '0006_untrackeduseragent'),
    ]

    operations = [
        migrations.CreateModel(
            name='BannedIP',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ip_address', models.GenericIPAddressField(help_text='The IP address that should be banned', verbose_name='IP Address')),
            ],
            options={
                'verbose_name': 'Banned IP',
                'verbose_name_plural': 'Banned IPs',
                'ordering': ('ip_address',),
            },
        ),
    ]



from django.test import TestCase

from isc_common.auth.models.user import User
from isc_common.auth.models.widgets_trees import Widgets_trees


class Test_Permission(TestCase):

    def setUp(self):
        self.user = User.objects.get(username='admin')
        self.widget = Widgets_trees.objects.get(id_widget='MainNitelArchive_DataViewSS')

    def test_1(self):
        grouops = self.user.usergroup.all()
        print('Hello World !!!')



from django.urls import path

from tracker.views import messages

urlpatterns = [

    path('Messages/Fetch/', messages.Messages_Fetch),
    path('Messages/Fetch4Chat/', messages.Messages_Fetch4Chat),
    path('Messages/Add', messages.Messages_Add),
    path('Messages/Add_AutoError', messages.Messages_Add_Auto_Error),
    path('Messages/Update', messages.Messages_Update),
    path('Messages/Lookup/', messages.Messages_Lookup),
    path('Messages/Remove', messages.Messages_Remove),
    path('Messages/Info/', messages.Messages_Info),

]



from django.db import connection


def get_deq_next_value(seq_name):
    cursor = connection.cursor()
    cursor.execute(f"select nextval('{seq_name}')")
    result, = cursor.fetchone()
    return result



import re

from django.conf import settings

# this is not intended to be an all-knowing IP address regex
IP_RE = re.compile('\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}')


def get_ip(request):
    """
    Retrieves the remote IP address from the request data.  If the user is
    behind a proxy, they may have a comma-separated list of IP addresses, so
    we need to account for that.  In such a case, only the first IP in the
    list will be retrieved.  Also, some hosts that use a proxy will put the
    REMOTE_ADDR into HTTP_X_FORWARDED_FOR.  This will handle pulling back the
    IP from the proper place.
    """

    # if neither header contain a value, just use local loopback
    ip_address = request.META.get('HTTP_X_FORWARDED_FOR', request.META.get('REMOTE_ADDR', '127.0.0.1'))
    if ip_address:
        # make sure we have one and only one IP
        try:
            ip_address = IP_RE.match(ip_address)
            if ip_address:
                ip_address = ip_address.group(0)
            else:
                # no IP, probably from some dirty proxy or other device
                # throw in some bogus IP
                ip_address = '10.0.0.1'
        except IndexError:
            pass

    return ip_address


def get_timeout():
    """
    Gets any specified timeout from the settings file, or use 10 minutes by
    default
    """
    return getattr(settings, 'TRACKING_TIMEOUT', 10)


def get_cleanup_timeout():
    """
    Gets any specified visitor clean-up timeout from the settings file, or
    use 24 hours by default
    """
    return getattr(settings, 'TRACKING_CLEANUP_TIMEOUT', 24)



from isc_common.auth.http.PermissionRequest import PermissionRequest
from isc_common.http.DSResponse import JsonResponseWithException
from isc_common.http.response import JsonResponse


@JsonResponseWithException(printing=False)
def permission_view(request):
    return JsonResponse(PermissionRequest(request).response)



import logging
import os

from django.conf import settings
from django.db.models import BinaryField, BigIntegerField, CharField, TextField
from django.forms import model_to_dict
from isc_common import delAttr
from isc_common.fields.files import FileFieldEx
from isc_common.fields.name_field import NameField
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel

logger = logging.getLogger(__name__)

class CryptoQuerySet(CommonManagetWithLookUpFieldsQuerySet):

    def delete(self):
        for item in self:
            old_file_store = item.file_store
            file_path = item.attfile.name

            if old_file_store:
                file_path = file_path.replace(old_file_store, settings.FILES_STORE)

            if isinstance(settings.REPLACE_FILE_PATH, dict):
                for key, value in settings.REPLACE_FILE_PATH.items():
                    file_path = file_path.replace(key, value)

            if file_path:
                if os.altsep:
                    file_path = file_path.replace(os.altsep, os.sep)

                if os.path.exists(file_path):
                    res = os.remove(file_path)
                    logger.debug(f'Removed file: {file_path}')
                else:
                    logger.warning(f'Removed file: {file_path} not finded.')
        return super().delete()


class CryptoManager(CommonManagetWithLookUpFieldsManager):
    def createFromRequest(self, request, function=None):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        for key in data:
            if key.find('__') != -1:
                delAttr(_data, key)
        delAttr(_data, 'form')
        if data.get('id') or not data.get('real_name'):
            delAttr(_data, 'id')
            res = super().filter(id=data.get('id')).update(**_data)
            res = model_to_dict(res[0])
            delAttr(res, 'attfile')
            delAttr(res, 'form')
            data.update(res)
        return data

    def updateFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        delAttr(data, 'form')
        super().filter(id=request.get_id()).update(**data)
        return data

    def get_queryset(self):
        return CryptoQuerySet(self.model, using=self._db)


class Crypto_file(AuditModel):
    format = NameField(verbose_name='Формат файла')
    mime_type = NameField(verbose_name='MIME тип файла файла', null=True, blank=True, default=None)
    size = BigIntegerField(verbose_name='Размер фала', default=0)
    real_name = TextField(verbose_name='Первоначальное имя файла', null=True, blank=True, default=None)
    key = BinaryField(max_length=200, null=True, blank=True)
    attfile = FileFieldEx(verbose_name='Файл', max_length=255, null=True, blank=True, default=None)
    file_store = CharField(verbose_name='Каталог хранения файла', max_length=255, null=True, blank=True)

    object = CryptoManager()

    def __str__(self):
        return f"{self.real_name}"

    class Meta:
        abstract = True



# Generated by Django 2.1.7 on 2019-02-16 20:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0002_rec_admin_group'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='user',
            name='permissions',
        ),
        migrations.RemoveField(
            model_name='usergroup',
            name='permissions',
        ),
    ]



import logging

from django.core.management import BaseCommand
from django.db import connection

from isc_common.common.mat_views import create_table

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('--tablename', type=str)

    def handle(self, *args, **options):
        tablename = options.get('tablename')

        create_table(
            sql_str=f'''select * from {tablename}_view''',
            table_name=f'''{tablename}_tbl''',
            primary_key=['id', 'location_id']
        )



import os
from typing import Text
from uuid import UUID


def getOrElse(default, value):
    if value:
        return value
    else:
        return default


blue = 'blue'
closed = 'closed'
create = 'create'
deleted = 'deleted'
doing = 'doing'
green = 'green'
made = 'made'
new = 'new'
new_man = 'new_man'
red = 'red'
restarted = 'restarted'
started = 'started'
transferred = 'transferred'
update = 'update'
value_odd = 'value_odd'


def blinkString(text, blink=True, color="black", bold=False) -> Text:
    if blink:
        res = f'<div class="blink"><strong><font color="{color}"</font>{text}</strong></div>'
    else:
        res = f'<div><strong><font color="{color}"</font>{text}</strong></div>'

    if bold == True:
        return f'<b>{res}</b>'
    else:
        return res


def uuid4():
    return str(UUID(bytes=os.urandom(16), version=4)).upper().replace('-', '_')



from django.urls import path

from history.views import history

urlpatterns = [

    path('History/Fetch/', history.History_Fetch),
    path('History/FetchGrouped/', history.History_FetchGrouped),
    path('History/Add', history.History_Add),
    path('History/Update', history.History_Update),
    path('History/Remove', history.History_Remove),
    path('History/Lookup/', history.History_Lookup),
    path('History/Info/', history.History_Info),

]



import logging

from django.db.models import JSONField
from django.forms import model_to_dict

from isc_common.auth.models.user_permission import User_permission
from isc_common.auth.models.usergroup_permission import Usergroup_permission
from isc_common.fields.code_field import CodeField
from isc_common.fields.description_field import DescriptionField
from isc_common.fields.name_field import NameField
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditManager
from isc_common.models.base_ref import Hierarcy

logger = logging.getLogger(__name__)


class Widgets_treesManager(AuditManager):
    def createFromRequest(self, request, printRequest=False):
        from isc_common.auth.models.usergroup import UserGroup

        from isc_common.auth.models.user import User

        request = DSRequest(request=request)
        id = request.json.get('id')
        table_name = request.json.get('table_name')
        mode = request.json.get('mode')

        data = request.get_data()
        if data.get('id_widget') == '/':
            data = data.get('children')[0]

        id_widget = data.get('id_widget')

        if id == None and mode == None and mode == None:
            res, create = super().update_or_create(**data)
            return res

        if table_name == 'widgets_trees':
            name = data.get('name')
            class_name = data.get('class_name')
            description = data.get('description')
            res, create = super().update_or_create(
                id_widget=id_widget,
                defaults=dict(
                    id_widget=id_widget,
                    name=name,
                    class_name=class_name,
                    description=description,
                    structure=data
                )
            )
            return model_to_dict(res)

        if table_name == 'user':
            user = User.objects.get(id=id)
            widget = Widgets_trees.objects.get(id_widget=id_widget)
            if mode == 'write':
                User_permission.objects.update_or_create(
                    user=user,
                    widget=widget,
                    defaults=dict(
                        permission=data
                    ))
            elif mode == 'remove':
                User_permission.objects.filter(
                    user=user,
                    widget=widget).delete()

            return dict()

        if table_name == 'user_group':
            usergroup = UserGroup.objects.get(id=id)
            widget = Widgets_trees.objects.get(id_widget=id_widget)

            if mode == 'write':
                Usergroup_permission.objects.update_or_create(
                    usergroup=usergroup,
                    widget=widget,
                    defaults=dict(
                        permission=data
                    ))
            elif mode == 'remove':
                Usergroup_permission.objects.filter(
                    usergroup=usergroup,
                    widget=widget).delete()

            return dict()

        return dict()

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'id_widget': record.id_widget,
            'name': record.name,
            'class_name': record.class_name,
            'description': record.description,
            'parent_id': record.parent.id if record.parent else None,
        }
        return res


class Widgets_trees(Hierarcy):
    def __str__(self):
        return self.id_widget

    id_widget = CodeField(unique=True)
    name = NameField()
    class_name = NameField()
    description = DescriptionField()
    structure = JSONField(default=dict)

    objects = Widgets_treesManager()

    def __str__(self):
        return f"id: {self.id_widget} name:{self.name_widget}, class_name:{self.class_name}"

    class Meta:
        verbose_name = 'Таблица сохранения деревьев виджетов объектов системы'



from django.utils.translation import ugettext_lazy as _

import logging

from isc_common.models.base_ref import BaseRef, BaseRefManager, BaseRefQuerySet

logger = logging.getLogger(__name__)


class Messages_importanceQuerySet(BaseRefQuerySet):
    pass


class Messages_importanceManager(BaseRefManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,
        }
        return res

    def get_queryset(self):
        return Messages_importanceQuerySet(self.model, using=self._db)


class Messages_importance(BaseRef):
    objects = Messages_importanceManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Важность дел'



import os


class Icons:
    _icons = dict(
        abonents='abonents.png',
        access='access.png',
        accounting='accounting.png',
        accounts='accounts.png',
        Actions_document_edit_icon='Actions-document-edit-icon.png',
        Actions_insert_link_icon='Actions-insert-link-icon.png',
        add='add.png',
        admin='admin.png',
        admin_User='admin_User.png',
        admin_UserGroup='admin_UserGroup.png',
        approved='approved.png',
        attach='attach.png',
        attachDisable='attach_Disable.png',
        attatch='attach.png',
        calc='calc.png',
        calculator='calculator.png',
        cancel='cancel.png',
        card='card.png',
        cards='cards.png',
        closeProgram='Windows-Close-Program-icon.png',
        common_ContractorGroup='common_ContractorGroup.png',
        communication='communication.png',
        company='company.png',
        completed='completed.png',
        contracts='contracts.png',
        copy='copy.png',
        copy_icon='copy.png',
        copy_value='copy_value.png',
        copyProduct='copyproduct.png',
        customer='customer.png',
        data='data.png',
        delete_Column='Delete_Column.png',
        delete_icon='Delete-icon.png',
        detail='detail.png',
        development='development.png',
        dictionary='dictionary.png',
        disabled='disabled.png',
        distribution='distribution.png',
        doccats='doccats.png',
        docitem='docitem.png',
        docizv='docizv.png',
        docizvtype='docizvtype.png',
        docliter='docliter.png',
        doctypes='doctypes.png',
        document='Document-icon.png',
        downloads='downloads.png',
        drawing='drawing.png',
        dvd='dvd.png',
        ellipsis='ellipsis.png',
        ellipsis_Disabled='ellipsis_Disabled.png',
        emulateOutMessage='emulateOutMessage.png',
        error='error.png',
        export='export.png',
        exportDB='exportDB.png',
        filter='filter.png',
        folder1='folder1.png',
        font='font.png',
        format='format.png',
        gradient='gpadient.png',
        grouping='grouping.png',
        groupitem='groupitem.png',
        guid='guid.png',
        Help_icon='Help-icon.png',
        hide='hide.png',
        history='history.png',
        iconAdd='Add_Column.png',
        iconConstructor='Start-Menu-icon.png',
        iconControl='ControlReading.png',
        iconDiscard='cancel-icon.png',
        iconEdit='edit.png',
        iconEllipsis='ellipsis.png',
        iconFile='file.png',
        iconFolder='folder.png',
        iconFunctions='function-selection.png',
        iconNew='Actions-insert-link-icon.png',
        iconOpenFolder='openFolder.png',
        iconRefresh='Refresh.png',
        iconSave='Save-icon.png',
        iconStatistic='systemservice_SeqGenerator.png',
        iconStatistiq='global_manager.png',
        iconTreeNode='document_plain_new.png',
        iconUnknown='unknown.png',
        info='info.png',
        informica='informica.ico',
        ini='ini.png',
        insert='insert.png',
        inventory='inventory.png',
        json_gear='json-gear.png',
        jsonGear='json-gear.png',
        lists='lists.png',
        load='load.png',
        login='login.png',
        mailing='mailing.png',
        material='material.png',
        menu='menu.png',
        message='message.png',
        minus='minus.png',
        mtype='mtype.png',
        mvid='mvid.png',
        node='node.png',
        notice='notice.png',
        notready='notReady.png',
        ok='ok.png',
        organization='organization.png',
        password='password.png',
        pdf='file_pdf.png',
        piz_lico='piz_lico.png',
        planing='planing.png',
        png='file_png.png',
        power='power.png',
        product='product.png',
        production='production.png',
        properties='properties.png',
        ready='ready.png',
        recorder='recorder.png',
        ref='ref.png',
        ref_RefRefs='ref_RefRefs.png',
        reports='reports.png',
        request='request.png',
        sales='sales.png',
        save='save.png',
        send='send.png',
        settings='settings.png',
        shadow='shadow.png',
        show_all='show-all.png',
        softStop='softStop.png',
        start='start.png',
        state='state.png',
        status='status.png',
        statusversion='statusversion.png',
        stocks='stocks.png',
        stoped='stoped.png',
        store='store.png',
        structure='structure.png',
        systemservice='systemservice.png',
        tasks='tasks.png',
        template='template.png',
        theme='theme.png',
        tiff='file_tiff.png',
        tools='tools.png',
        type='type.png',
        upload='upload.png',
        ur_entity='ur_entity.png',
        userposion='userposiion.png',
        visible='visible.png',
        Workflow='Workflow.png',
        zapros='zapros.png',
    )

    def __init__(self, *args, **kwargs):
        self.base_path_2_icons = kwargs.get('base_path_2_icons')

    def get(self, name):
        app_dir = f'{self.base_path_2_icons}{os.sep}{self._icons.get(name)}'
        if not os.path.exists(app_dir):
            raise Exception(f'Path: {app_dir} not exists.')
        return self._icons.get(name)



# Generated by Django 3.0.8 on 2020-08-03 14:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0080_auto_20200208_1350'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='messages',
            name='Messages_unique_constraint_0',
        ),
        migrations.RemoveConstraint(
            model_name='messages',
            name='Messages_unique_constraint_1',
        ),
        migrations.AddConstraint(
            model_name='messages',
            constraint=models.UniqueConstraint(condition=models.Q(('guid', None), ('importance', None), ('message', None)), fields=('checksum', 'state', 'theme', 'to_whom', 'user'), name='Messages_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='messages',
            constraint=models.UniqueConstraint(condition=models.Q(('importance', None), ('message', None)), fields=('checksum', 'guid', 'state', 'theme', 'to_whom', 'user'), name='Messages_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='messages',
            constraint=models.UniqueConstraint(condition=models.Q(('guid', None), ('message', None)), fields=('checksum', 'importance', 'state', 'theme', 'to_whom', 'user'), name='Messages_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='messages',
            constraint=models.UniqueConstraint(condition=models.Q(message=None), fields=('checksum', 'guid', 'importance', 'state', 'theme', 'to_whom', 'user'), name='Messages_unique_constraint_3'),
        ),
        migrations.AddConstraint(
            model_name='messages',
            constraint=models.UniqueConstraint(condition=models.Q(('guid', None), ('importance', None)), fields=('checksum', 'message', 'state', 'theme', 'to_whom', 'user'), name='Messages_unique_constraint_4'),
        ),
        migrations.AddConstraint(
            model_name='messages',
            constraint=models.UniqueConstraint(condition=models.Q(importance=None), fields=('checksum', 'guid', 'message', 'state', 'theme', 'to_whom', 'user'), name='Messages_unique_constraint_5'),
        ),
        migrations.AddConstraint(
            model_name='messages',
            constraint=models.UniqueConstraint(condition=models.Q(guid=None), fields=('checksum', 'importance', 'message', 'state', 'theme', 'to_whom', 'user'), name='Messages_unique_constraint_6'),
        ),
        migrations.AddConstraint(
            model_name='messages',
            constraint=models.UniqueConstraint(fields=('checksum', 'guid', 'importance', 'message', 'state', 'theme', 'to_whom', 'user'), name='Messages_unique_constraint_7'),
        ),
    ]



import logging

from django.core.management import BaseCommand

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = 'Создание модели календаря.'

    def add_arguments(self, parser):
        parser.add_argument('--calendar_id', type=int)
        parser.add_argument('--user_id', type=int)

    def handle(self, *args, **options):
        from clndr.time_line import Timeline

        calendar_id = options.get('calendar_id')
        user_id = options.get('user_id')

        logger.debug(self.help)

        data = dict(user_id=user_id)
        shiftsList = Timeline(calendar_id=calendar_id, logger=logger).build(data=data)
        shiftsList.write_2_file(f'dest_debug/calendar_{calendar_id}.js')
        json = shiftsList.to_json()
        print(len(json))



# Generated by Django 2.2.5 on 2019-09-05 05:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0057_auto_20190904_1843'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='messages',
            options={'ordering': ('lastmodified',), 'verbose_name': 'Сообщения'},
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-16 18:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('history', '0011_auto_20191116_1318'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='visitor',
            unique_together={('username', 'ip_address')},
        ),
    ]



from isc_common.auth.http.get_permission_request import GetPermissionRequest
from isc_common.http.DSResponse import JsonResponseWithException
from isc_common.http.response import JsonResponse


@JsonResponseWithException(printing=False)
def get_permission_view(request):
    return JsonResponse(GetPermissionRequest(request).response)



from django.urls import path

from clndr.views import holidays

urlpatterns = [

    path('Holidays/Fetch/', holidays.Holidays_Fetch),
    path('Holidays/Add', holidays.Holidays_Add),
    path('Holidays/Update', holidays.Holidays_Update),
    path('Holidays/Remove', holidays.Holidays_Remove),
    path('Holidays/Lookup/', holidays.Holidays_Lookup),
    path('Holidays/Info/', holidays.Holidays_Info),
    path('Holidays/Copy', holidays.Holidays_Copy),

]



from django.db.models import CharField
from isc_common.fields import Field


class PhoneField(CharField, Field):

    def get_internal_type(self):
        return "CharField"

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('max_length', 50)
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        del kwargs["max_length"]
        return name, path, args, kwargs



from enum import IntEnum

from django.db.models import QuerySet


class Class:
    def __init__(self):
        self._excluded_keys = ["_excluded_keys"]

    def querySetToDict(self, vqs):
        return [item for item in vqs]

    @property
    def dict(self):
        return dict((key, value) for (key, value) in self.__dict__.items() if value is not None and key not in self._excluded_keys)


class RPCResponseConstant(IntEnum):
    statusSuccess = 0
    statusOffline = 1
    statusFailure = -1
    statusValidationError = -4
    statusLoginIncorrect = -5
    statusMaxLogginAttemptsExceeded = -6
    statusLoginRequired = -7
    statusLoginSuccess = -8
    statusUpdateWithoutPkError = -9
    statusTransactionFailed = -10
    statusTransportError = -90
    statusUnknownHostError = -91
    statusConnetionResetError = -92
    statusServerTimeout = -100
    statusNoFileAttached = -101


class RPCResponse(Class):

    def __init__(self, clientContext=None, data=None, httpHeaders=None, httpResponseCode=None, httpResponseText=None, status=0, transactionNum=None):
        Class.__init__(self)
        self.clientContext = clientContext
        if isinstance(data, QuerySet):
            self.data = self.querySetToDict(data)
        elif isinstance(data, list):
            self.data = data
        elif isinstance(data, dict):
            self.data = data
        self.httpHeaders = httpHeaders
        self.httpResponseCode = httpResponseCode
        self.httpResponseText = httpResponseText
        self.status = status
        self.transactionNum = transactionNum



import logging

from django.db.models.query import RawQuerySet

from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditManager, AuditQuerySet

logger = logging.getLogger(__name__)


class TreeAuditModelQuerySet(AuditQuerySet):
    check_dliting = True

    def delete(self):
        if self.check_dliting:
            raise Exception('Удаление без проверки !!!!')
        return super().delete()

    def get_descendants(self,
                        id=None,
                        start=None,
                        end=None,
                        child_id='child_id',
                        parent_id='parent_id',
                        child_parent_value=None,
                        include_self=True,
                        where_clause='',
                        order_by_clause='',
                        distinct='',
                        sql_text=None,
                        fields='*',
                        limit=None
                        ) -> RawQuerySet:
        db_name = self.model._meta.db_table

        if limit != None:
            if not isinstance(limit, int):
                raise Exception(f'limit must be int.')
            limit = f'limit {limit}'
        else:
            limit = ''

        if id != None:
            if isinstance(id, int):
                id = tuple([id])
            elif isinstance(id, str):
                id = int(id)
            elif not isinstance(id, tuple):
                raise Exception(f'id must be list or int')

        if sql_text != None:
            res = super().raw(sql_text)
        elif id != None:
            sql_text = f'''WITH RECURSIVE r AS (
                                select s.* from (SELECT {fields}, 1 AS level
                                FROM {db_name}
                                WHERE {child_id if include_self else parent_id} IN %s
                                      {"and " + where_clause.replace("where", "") if where_clause else ""} {limit}) as s  
    
                                union all
    
                                SELECT {db_name}.{fields}, r.level + 1 AS level
                                FROM {db_name}
                                    JOIN r
                                ON {db_name}.{parent_id} = r.{child_id})
    
                            select {distinct} {fields} from r {where_clause} {order_by_clause} limit %s offset %s
                        '''
            # logger.debug(f'sql: {sql_text}')
            res = super().raw(sql_text, params=((id,), end, start))
        else:
            child_parent_value_str = f' = {child_parent_value}'
            sql_text = f'''WITH RECURSIVE r AS (
                                            SELECT *, 1 AS level
                                            FROM {db_name}
                                            WHERE {child_id if include_self else parent_id} {"IS NULL" if child_parent_value == None else child_parent_value_str}
                                                  {"and " + where_clause.replace("where", "") if where_clause else ""}  

                                            union all

                                            SELECT {db_name}.*, r.level + 1 AS level
                                            FROM {db_name}
                                                JOIN r
                                            ON {db_name}.{parent_id} = r.{child_id})

                                        select {fields} from r {where_clause} {order_by_clause} limit %s offset %s
                                    '''
            # logger.debug(f'sql: {sql_text}')
            res = super().raw(sql_text, params=(end, start))

        return res

    def get_parents(self,
                    id=None,
                    start=None,
                    end=None,
                    child_id='child_id',
                    parent_id='parent_id',
                    include_self=True,
                    where_clause='',
                    order_by_clause='',
                    sql_text=None,
                    fields='*'
                    ) -> RawQuerySet:
        db_name = self.model._meta.db_table

        if isinstance(id, int):
            id = tuple([id])
        elif not isinstance(id, tuple):
            raise Exception(f'id must be list or int')

        sql = f'''WITH RECURSIVE r AS (
                            SELECT *, 1 AS level
                            FROM {db_name}
                            WHERE {parent_id if include_self else child_id} IN %s

                            union all

                            SELECT {db_name}.*, r.level + 1 AS level
                            FROM {db_name}
                                JOIN r
                            ON {db_name}.{child_id} = r.{parent_id})

                        select {fields} from r {where_clause} {order_by_clause} limit %s offset %s
                        '''

        logger.debug(f'sql: {sql}')
        res = super().raw(sql, params=((id,), end, start))

        return res

    def get_descendants_count(self,
                              id='id',
                              limit=None,
                              limit1=None,
                              offset=None,
                              child_id='child_id',
                              parent_id='parent_id',
                              child_parent_value=None,
                              include_self=True,
                              where_clause='',
                              distinct='',
                              sql_text=None,
                              fields='*',
                              ) -> int:
        return len(self.get_descendants(
            id=id,
            start=offset,
            end=limit,
            child_id=child_id,
            parent_id=parent_id,
            child_parent_value=child_parent_value,
            include_self=include_self,
            where_clause=where_clause,
            distinct=distinct,
            sql_text=sql_text,
            fields=fields,
            limit=limit1
        ))


class TreeAuditModelManager(AuditManager):
    def get_range_rows1(self, request, function=None, distinct_field_names=None):
        request = DSRequest(request=request)
        self.alive_only = request.alive_only
        self.enabledAll = request.enabledAll
        res = self.get_queryset().get_range_rows(start=request.startRow, end=request.endRow, function=function, distinct_field_names=distinct_field_names, json=request.json)
        return res

    def get_descendants(self,
                        id=None,
                        limit=None,
                        limit1=None,
                        offset=None,
                        child_id='child_id',
                        parent_id='parent_id',
                        child_parent_value=None,
                        include_self=True,
                        where_clause='',
                        order_by_clause='',
                        distinct='',
                        fields='*',
                        sql_text=None,
                        ) -> RawQuerySet:
        return self.get_queryset().get_descendants(
            id=id,
            start=offset,
            end=limit,
            child_id=child_id,
            parent_id=parent_id,
            child_parent_value=child_parent_value,
            include_self=include_self,
            where_clause=where_clause,
            order_by_clause=order_by_clause,
            distinct=distinct,
            fields=fields, sql_text=sql_text,
            limit=limit1
        )

    def get_parents(self,
                    id=None,
                    limit=None,
                    offset=None,
                    child_id='child_id',
                    parent_id='parent_id',
                    include_self=True,
                    where_clause='',
                    order_by_clause='',
                    fields='*',
                    sql_text=None,
                    ) -> RawQuerySet:
        return self.get_queryset().get_parents(
            id=id,
            start=offset,
            end=limit,
            child_id=child_id,
            parent_id=parent_id,
            include_self=include_self,
            where_clause=where_clause,
            order_by_clause=order_by_clause,
            fields=fields,
            sql_text=sql_text
        )

    def get_descendants_count(self,
                              id=None,
                              limit=None,
                              limit1=None,
                              offset=None,
                              child_id='child_id',
                              parent_id='parent_id',
                              child_parent_value=None,
                              include_self=True,
                              where_clause='',
                              distinct='',
                              sql_text=None,
                              fields='*'
                              ) -> int:
        return self.get_queryset().get_descendants_count(
            id=id,
            limit=limit,
            offset=offset,
            child_id=child_id,
            parent_id=parent_id,
            child_parent_value=child_parent_value,
            include_self=include_self,
            distinct=distinct,
            where_clause=where_clause,
            sql_text=sql_text,
            fields=fields,
            limit1=limit1
        )

    def get_queryset(self):
        return TreeAuditModelQuerySet(self.model, using=self._db)



# Generated by Django 2.1.7 on 2019-03-20 10:15

import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

import isc_common.fields.code_field
import isc_common.fields.related


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0012_auto_20190307_0100'),
    ]

    operations = [
        migrations.CreateModel(
            name='History',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('modified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('model', isc_common.fields.code_field.CodeField(verbose_name='Наименование таблицы')),
                ('data', django.db.models.JSONField(verbose_name='Данные учавствующие в транзакции')),
                ('old_data', django.db.models.JSONField(verbose_name='Данные учавствующие в транзакции (старые)')),
                ('description', models.TextField(verbose_name='Описание транзакции')),
                ('user', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='Пользователь сделавший изменение')),
            ],
            options={
                'verbose_name': 'History',
            },
        ),
    ]



import logging

from isc_common import setAttr, delAttr
from isc_common.http.RPCRequest import RPCRequest
from isc_common.json import BytesToJson

logger = logging.getLogger(__name__)


class RequestData:
    def __init__(self, _dict=None, request=None):
        if request:
            self.__data_dict__ = request.get_data()
        elif _dict:
            self.__data_dict__ = _dict
        else:
            self.__data_dict__ = dict()
        super()

    def getDataOfKey(self, key, default=None):
        return self.__data_dict__.get(key, default)

    def update(self, key, value):
        self.__data_dict__.update((key, value))
        return RequestData(self.__data_dict__)

    def getDataWithOutField(self, list_fields):
        if isinstance(list_fields, list):
            return RequestData({key: self.__data_dict__[key] for key in self.__data_dict__ if not key in list_fields})
        else:
            return RequestData(self.__data_dict__)

    def getDataNotContain(self, *sub_strs):
        res = RequestData()
        for sub in sub_strs:
            self.__data_dict__ = {key: self.__data_dict__[key] for key in self.__data_dict__ if not str(key).__contains__(sub)}
        return RequestData(self.__data_dict__)

    def dict(self):
        return self.__data_dict__


class DSRequest(RPCRequest):
    alive_only = None
    begdate = None
    dataPageSize = 75
    drawAheadRatio = 1.2
    enabledAll = None
    enddate = None
    endRow = None
    json = None
    startRow = None
    tag = None
    httpHeaders = None
    visibleMode = 'none'

    _user = None

    @property
    def user(self):
        from isc_common.auth.models.user import User
        if self._user == None:
            if self.httpHeaders != None and self.httpHeaders.USER_ID:
                self._user = User.objects.get(id=self.httpHeaders.USER_ID)
                return self._user
            else:
                if self.transaction != None:
                    operations = self.transaction.operations
                    if isinstance(operations, list):
                        for operation in operations:
                            data = operation.data
                            if data.httpHeaders != None and data.httpHeaders.USER_ID != None:
                                self._user = User.objects.get(id=data.httpHeaders.USER_ID)
                                return self._user

                raise Exception('user not enable type')
        return self._user

    @property
    def user_id(self):
        return self.user.id

    @property
    def fio(self):
        return self.user.get_short_name

    @property
    def username(self):
        return self.user.username

    @property
    def is_admin(self):
        return self.user.is_admin

    @property
    def is_develop(self):
        return self.user.is_develop

    def set_user_data(self, user):
        from isc_common.auth.models.user import User
        if isinstance(user, int):
            self._user = User.objects.get(id=user)
        elif isinstance(user, str):
            self._user = User.objects.get(username=user)
        else:
            raise Exception('user not enable type')

    def __init__(self, request):

        if request != None and request.body != b'':
            self.json = BytesToJson(request.body)
            if isinstance(self.json, dict) and len(self.json) > 0:
                data = self.json.get('data')
                if isinstance(data, dict):
                    httpHeaders = data.get('httpHeaders')
                    if isinstance(httpHeaders, dict):

                        if httpHeaders.get('WS_CHANNEL') != None:
                            self.ws_channel = httpHeaders.get('WS_CHANNEL')

                        if httpHeaders.get('WS_PORT') != None:
                            self.ws_port = httpHeaders.get('WS_PORT')

                        if httpHeaders.get('HOST') != None:
                            self.host = httpHeaders.get('HOST')

                        if httpHeaders.get('USERNAME') != None:
                            self.set_user_data(user=httpHeaders.get('USERNAME'))

                        if httpHeaders.get('USER_ID') != None:
                            self.set_user_data(user=httpHeaders.get('USER_ID'))

                        delAttr(self.json.get('data'), 'httpHeaders')
                    else:
                        if data.get('user_id') != None:
                            self.set_user_data(user=data.get('user_id'))
                else:
                    login = self.json.get('login')
                    if login != None:
                        self.set_user_data(user=login)
                    else:
                        httpHeaders = self.json.get('httpHeaders')
                        if httpHeaders != None:
                            self.set_user_data(user=httpHeaders.get('USER_ID'))

        RPCRequest.__init__(self, self.json)

    def get_data(self, excluded_keys=['grid']):
        res = None
        if isinstance(self.json, dict):
            data = self.json.get('data')
            if isinstance(data, dict):
                res = data
            elif isinstance(data, str):
                return data
            else:
                res = self.json

            res = dict((key, value) for (key, value) in res.items() if not key.startswith('_') and key not in excluded_keys)
        return res

    def get_all_data(self, excluded_keys=['grid']):
        res = None
        if isinstance(self.json, dict):
            data = self.json.get('data')
            if isinstance(data, dict):
                res = data
            elif isinstance(data, str):
                return data
            else:
                res = self.json

        return res

    def set_data(self, data):
        if isinstance(data, dict):
            setAttr(self.json, 'data', data)

    def set_criteria(self, criteria, excluded_keys=['grid']):
        data = self.get_all_data(excluded_keys=excluded_keys)
        if isinstance(data, dict):
            setAttr(data, 'criteria', criteria)

    def get_criteria(self, excluded_keys=['grid']):
        data = self.get_data(excluded_keys=excluded_keys)
        criteria = data.get('criteria')
        if isinstance(criteria, list):
            return criteria
        else:
            return []

    def get_data_array(self, excluded_keys=['grid']):
        res = None
        if isinstance(self.json, dict):
            transaction = self.json.get('transaction')
            if transaction:
                operations = transaction.get('operations')
            else:
                return (False, [self.get_data()])

            res = (True, [dict((key, value) for (key, value) in operation.items() if not str(key).startswith('_') and key not in excluded_keys) for operation in operations])
            # res = (True, [dict((key, value) for (key, value) in operation.items() if value is not None and not str(key).startswith('_') and key not in excluded_keys) for operation in operations])
        return res

    def get_oldValues(self, excluded_keys=['grid']):
        res = None
        if isinstance(self.json, dict):
            if self.json.get('oldValues', None) and isinstance(self.json.get('oldValues', None), dict):
                res = self.json.get('oldValues')
            else:
                res = self.json

            res = dict((key, value) for (key, value) in res.items() if not str(key).startswith('_') and key not in excluded_keys)
            # res = dict((key, value) for (key, value) in res.items() if value is not None and not str(key).startswith('_') and key not in excluded_keys)
        return res

    def get_id(self):
        return self.get_data().get('id', self.get_oldValues().get('id'))

    def get_operationtype(self):
        return self.json.get('operationType')

    def get_username(self):
        return self.get_data().get('username')

    def get_ids(self):
        multi, data_array = self.get_data_array()
        if data_array:
            if multi:
                ids = [record.get('data').get('id') for record in data_array]
            else:
                ids = [record.get('data').get('id') if record.get('data') else record.get('id') for record in data_array]
        else:
            ids = [0]
        return [id for id in ids if id is not None]

    def get_tuple_ids(self):
        multi, data_array = self.get_data_array()
        if data_array:
            if multi:
                ids = [(record.get('data').get('id'), record.get('visibleMode', 'none')) for record in data_array]
            else:
                ids = [(record.get('data').get('id'), self.visibleMode) if record.get('data') else (record.get('id'), self.visibleMode) for record in data_array]
        else:
            ids = [(0, "none")]
        return ids

    def get_old_ids(self):
        multi, data_array = self.get_data_array()
        if data_array:
            if multi:
                ids = [record.get('oldValues').get('id') for record in data_array]
            else:
                ids = [record.get('oldValues').get('id') if record.get('oldValues') else record.get('id') for record in data_array]
        else:
            ids = [0]
        return [id for id in ids if id is not None]

    def __str__(self):
        return str(self.__dict__)



# Generated by Django 2.2.2 on 2019-06-10 14:36

import django.utils.timezone
from django.conf import settings
from django.db import migrations, models
from django.db.models import JSONField

import isc_common.fields.code_field
import isc_common.fields.related


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0025_widgets_trees_parent'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='history',
            name='deleted_at',
        ),
        migrations.RemoveField(
            model_name='history',
            name='deliting',
        ),
        migrations.RemoveField(
            model_name='history',
            name='description',
        ),
        migrations.RemoveField(
            model_name='history',
            name='editing',
        ),
        migrations.RemoveField(
            model_name='history',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='history',
            name='lastmodified',
        ),
        migrations.RemoveField(
            model_name='history',
            name='model',
        ),
        migrations.RemoveField(
            model_name='history',
            name='modified',
        ),
        migrations.RemoveField(
            model_name='history',
            name='old_data',
        ),
        migrations.AddField(
            model_name='history',
            name='date',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name='history',
            name='method',
            field=isc_common.fields.code_field.CodeField(),
        ),
        migrations.AddField(
            model_name='history',
            name='path',
            field=isc_common.fields.code_field.CodeField(),
        ),
        migrations.AlterField(
            model_name='history',
            name='data',
            field=django.db.models.JSONField(default=dict),
        ),
        migrations.AlterField(
            model_name='history',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='history',
            name='user',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-21 12:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0012_holidays_props'),
    ]

    operations = [
        migrations.AlterField(
            model_name='calendars',
            name='begindate',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Начало'),
        ),
        migrations.AlterField(
            model_name='calendars',
            name='enddate',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Конец'),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-16 10:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('history', '0005_visitor'),
    ]

    operations = [
        migrations.CreateModel(
            name='UntrackedUserAgent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('keyword', models.CharField(help_text='Part or all of a user-agent string.  For example, "Googlebot" here will be found in "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)" and that visitor will not be tracked.', max_length=255, verbose_name='keyword')),
            ],
            options={
                'verbose_name': 'Untracked User-Agent',
                'verbose_name_plural': 'Untracked User-Agents',
                'ordering': ('keyword',),
            },
        ),
    ]



# Generated by Django 2.2.5 on 2019-09-05 17:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('twits', '0004_auto_20190830_2250'),
    ]

    operations = [
        migrations.AddField(
            model_name='chat_user_user',
            name='common_id',
            field=models.PositiveIntegerField(default=None),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-19 12:05

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('twits', '0001_initial'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='chat_users',
            unique_together={('user', 'chat')},
        ),
    ]



from django.urls import path

from tracker.views import messages_files, messages

urlpatterns = [

    path('Messages_files/Fetch/', messages_files.Messages_files_Fetch),
    path('Messages_files/Add', messages_files.Messages_files_Add),
    path('Messages_files/Update', messages_files.Messages_files_Update),
    path('Messages_files/Remove', messages_files.Messages_files_Remove),
    path('Messages_files/Lookup/', messages_files.Messages_files_Lookup),
    path('Messages_files/Info/', messages_files.Messages_files_Info),
    path('Messages_files/Copy', messages_files.Messages_files_Copy),
    path('Messages_files/Upload', messages.Messages_UploadFile),
    path('Messages_files/Upload1', messages.Messages_UploadFile1),
    path('Messages_files/Confirm_reading', messages.Messages_Confirm_reading),
    path('Messages_files/Download/<int:id>/', messages.Message_DownloadFile),

]



from django.conf import settings
from django.db.models import FileField
from django.db.models.fields.files import FieldFile
from isc_common.fields import Field


class FieldFileEx(FieldFile, Field):
    def open(self, mode='rb'):
        self._require_file()
        if getattr(self, '_file', None) is None:
            if isinstance(settings.REPLACE_FILE_PATH, dict):
                for key, value in settings.REPLACE_FILE_PATH.items():
                    self.name = self.name.replace(key, value)
            self.file = self.storage.open(self.name, mode)
        else:
            self.file.open(mode)
        return self


class FileFieldEx(FileField):
    attr_class = FieldFileEx



import json
from json import JSONDecodeError

from isc_common.string import StrToBytes, BytesToStr


def BytesToJson(mybytes):
    try:
        mystring = BytesToStr(mybytes)
        _json = json.loads(mystring)
        return _json
    except JSONDecodeError:
        return dict()

def JsonToBytes(meyjson):
    mystring = json.dumps(meyjson)
    bytes = StrToBytes(mystring)
    return bytes



from django.urls import path

from isc_common.views import deleted_progresses

urlpatterns = [

    path('Deleted_progresses/Fetch/', deleted_progresses.Deleted_progresses_Fetch),
    path('Deleted_progresses/Add', deleted_progresses.Deleted_progresses_Add),
    path('Deleted_progresses/Update', deleted_progresses.Deleted_progresses_Update),
    path('Deleted_progresses/Remove', deleted_progresses.Deleted_progresses_Remove),
    path('Deleted_progresses/Lookup/', deleted_progresses.Deleted_progresses_Lookup),
    path('Deleted_progresses/Info/', deleted_progresses.Deleted_progresses_Info),
    path('Deleted_progresses/Copy', deleted_progresses.Deleted_progresses_Copy),

]



import datetime


def DateToStr(date, mask='%d.%m.%Y', hours=3):
    if date == None:
        return None
    if isinstance(date, str):
        date = StrToDate(date)
    return datetime.datetime.strftime(date + datetime.timedelta(hours=hours), mask)


def DateTimeToStr(date, mask='%d.%m.%Y %H:%M:%S', hours=0):
    if date == None:
        return None
    if isinstance(date, str):
        date = StrToDate(date)
    return datetime.datetime.strftime(date + datetime.timedelta(hours=hours), mask)


def manth_name(month):
    if month == '01':
        return 'Январь'
    if month == '02':
        return 'Февраль'
    if month == '03':
        return 'Март'
    if month == '04':
        return 'Апрель'
    if month == '05':
        return 'Май'
    if month == '06':
        return 'Июнь'
    if month == '07':
        return 'Июль'
    if month == '08':
        return 'Август'
    if month == '09':
        return 'Сентябрь'
    if month == '10':
        return 'Октябрь'
    if month == '11':
        return 'Ноябрь'
    if month == '12':
        return 'Декабрь'


# Format Code List
# The table below shows all the format codes that you can use.
#
# Directive	Meaning	Example
# %a	Abbreviated weekday name.	Sun, Mon, ...
# %A	Full weekday name.	Sunday, Monday, ...
# %w	Weekday as a decimal number.	0, 1, ..., 6
# %d	Day of the month as a zero-padded decimal.	01, 02, ..., 31
# %-d	Day of the month as a decimal number.	1, 2, ..., 30
# %b	Abbreviated month name.	Jan, Feb, ..., Dec
# %B	Full month name.	January, February, ...
# %m	Month as a zero-padded decimal number.	01, 02, ..., 12
# %-m	Month as a decimal number.	1, 2, ..., 12
# %y	Year without century as a zero-padded decimal number.	00, 01, ..., 99
# %-y	Year without century as a decimal number.	0, 1, ..., 99
# %Y	Year with century as a decimal number.	2013, 2019 etc.
# %H	Hour (24-hour clock) as a zero-padded decimal number.	00, 01, ..., 23
# %-H	Hour (24-hour clock) as a decimal number.	0, 1, ..., 23
# %I	Hour (12-hour clock) as a zero-padded decimal number.	01, 02, ..., 12
# %-I	Hour (12-hour clock) as a decimal number.	1, 2, ... 12
# %p	Locale’s AM or PM.	AM, PM
# %M	Minute as a zero-padded decimal number.	00, 01, ..., 59
# %-M	Minute as a decimal number.	0, 1, ..., 59
# %S	Second as a zero-padded decimal number.	00, 01, ..., 59
# %-S	Second as a decimal number.	0, 1, ..., 59
# %f	Microsecond as a decimal number, zero-padded on the left.	000000 - 999999
# %z	UTC offset in the form +HHMM or -HHMM.
# %Z	Time zone name.
# %j	Day of the year as a zero-padded decimal number.	001, 002, ..., 366
# %-j	Day of the year as a decimal number.	1, 2, ..., 366
# %U	Week number of the year (Sunday as the first day of the week). All days in a new year preceding the first Sunday are considered to be in week 0.	00, 01, ..., 53
# %W	Week number of the year (Monday as the first day of the week). All days in a new year preceding the first Monday are considered to be in week 0.	00, 01, ..., 53
# %c	Locale’s appropriate date and time representation.	Mon Sep 30 07:06:05 2013
# %x	Locale’s appropriate date representation.	09/30/13
# %X	Locale’s appropriate time representation.	07:06:05
# %%	A literal '%' character.	%

def StrToDate(value_str, mask='%Y-%m-%dT%H:%M:%S', hours=0):
    if value_str == None:
        return None

    if isinstance(value_str, datetime.date):
        return value_str

    if len(value_str.split('.')) == 2:
        value_str = value_str.split('.')[0]
    res = datetime.datetime.strptime(value_str, mask)
    # return res
    return res + datetime.timedelta(hours=hours)



import json
import logging
import math
import pprint
import sys
import traceback

from django.conf import settings
from django.db.models import QuerySet, Model
from django.forms import model_to_dict
from websocket import create_connection

from isc_common import setAttr, Wrapper
from isc_common.common.functions import Common
from isc_common.http.DSRequest import RequestData, DSRequest
from isc_common.http.RPCResponse import RPCResponse, RPCResponseConstant
from isc_common.http.response import JsonResponse

logger = logging.getLogger(__name__)


class Exception_WS_Enabled(Exception):
    pass


class BodyWrapper(Wrapper):
    data = None
    login = None
    json = None
    httpHeaders = None
    transaction = None


class Body:
    session = None

    def __init__(self, body, session):
        self.session = session
        if not isinstance(body, dict):
            raise Exception('body must by dict')

        self.body = BodyWrapper(**body)
        if self.body.transaction != None:
            operations = self.body.transaction.get('operations')
            if isinstance(operations, list):
                for operation in operations:
                    data = operation.get('data')
                    httpHeaders = data.get('httpHeaders')
                    if isinstance(httpHeaders, dict):
                        self.body.httpHeaders = httpHeaders
                        break

        if self.body.httpHeaders != None:
            self.body.httpHeaders = Wrapper(**self.body.httpHeaders)
        elif self.body.data != None and self.body.data.get('httpHeaders') != None:
            self.body.httpHeaders = Wrapper(**self.body.data.get('httpHeaders'))

    @property
    def login(self):
        from isc_common.auth.models.user import User
        if self.body.login != None:
            return self.body.login
        elif self.body.httpHeaders != None and self.body.httpHeaders.USER_ID:
            return User.objects.get(id=self.body.httpHeaders.USER_ID).username
        else:
            if self.session != None and self.session._session != None:
                username = self.session._session.get('ws_channel').split('_')[1]
                return username

            raise Exception('user not enable type')

def JsonWSResponseWithException(printing=False, printing_res=False):
    try:
        class JE:
            def __init__(self, func):
                self.func = func
                self.pp = pprint.PrettyPrinter(indent=4)

            def print(self, o, str=None):
                if str:
                    print(f"=========== Begin {str} ========================")
                else:
                    print(f"=========== Begin {o.__class__.__name__} ========================")
                if isinstance(o, dict):
                    self.pp.pprint(RequestData(o).getDataWithOutField(['pp', '_state']).dict())
                elif isinstance(o, object):
                    self.pp.pprint(RequestData(o.__dict__).getDataWithOutField(['pp', '_state']).dict())

                if str:
                    print(f"=========== End   {str} ========================")
                else:
                    print(f"=========== End   {o.__class__.__name__} ========================")
                print("\n")

            def __call__(self, *args, **kwargs):
                from isc_common.json import BytesToJson
                request = args[0]

                self.port = request.GET.get('ws_port')
                if self.port == None:
                    self.port = request.GET.get('port')
                    if self.port == None:
                        self.port = settings.WS_PORT
                        if self.port == None:
                            raise Exception_WS_Enabled(Common.arraund_error('Do not have ws_port'))

                self.host = request.GET.get('ws_host')
                if self.host == None:
                    self.host = request.GET.get('host')
                    if self.host == None:
                        self.host = settings.WS_HOST
                        if self.host == None:
                            raise Exception_WS_Enabled(Common.arraund_error('Do not have ws_host'))

                self.channel = request.GET.get('ws_channel')
                if self.channel == None:
                    body = BytesToJson(request.body)
                    body = Body(body, request.session)
                    if settings.WS_CHANNEL != None:
                        self.channel = f'''{settings.WS_CHANNEL}_{body.login}'''

                try:
                    if printing:
                        _request = DSRequest(request)
                        self.print(_request)
                        self.print(request.COOKIES, 'request.COOKIES')
                        self.print(request.GET, 'request.GET')
                        self.print(request.POST, 'request.POST')
                        self.print(request.META, 'request.META')

                    res = self.func(*args, **kwargs)
                    if printing_res:
                        self.print(res)

                    return res
                except Exception as ex:
                    exc_info = sys.exc_info()
                    stackTrace = traceback.format_exception(*exc_info)
                    message = str(ex)

                    try:
                        ws = create_connection(f"ws://{self.host}:{self.port}/ws/{self.channel}/")
                        ws.send(json.dumps(dict(message=message, stackTrace=stackTrace, type="error")))
                        ws.close()
                    except Exception as ex:

                        exc_info = sys.exc_info()
                        message = str(ex)
                        stackTrace = traceback.format_exception(*exc_info)

                        logging.error(message)

                        for x in stackTrace:
                            logger.error(x)

                        del exc_info

                    for x in stackTrace:
                        logger.error(x)

                    return JsonResponse(DSResponseOk().response)

        return JE
    except Exception_WS_Enabled:
        return JsonResponseWithExceptionOld(printing=printing, printing_res=printing_res)


def JsonResponseWithException(printing=False, printing_res=False):
    return JsonWSResponseWithException(printing=printing, printing_res=printing_res)


def JsonResponseWithExceptionOld(printing=False, printing_res=False):
    class JE:
        def __init__(self, func):
            self.func = func
            self.pp = pprint.PrettyPrinter(indent=4)

        def print(self, o, str=None):
            if str:
                print(f"=========== Begin {str} ========================")
            else:
                print(f"=========== Begin {o.__class__.__name__} ========================")
            if isinstance(o, dict):
                self.pp.pprint(RequestData(o).getDataWithOutField(['pp', '_state']).dict())
            elif isinstance(o, object):
                self.pp.pprint(RequestData(o.__dict__).getDataWithOutField(['pp', '_state']).dict())

            if str:
                print(f"=========== End   {str} ========================")
            else:
                print(f"=========== End   {o.__class__.__name__} ========================")
            print("\n")

        def __call__(self, *args, **kwargs):
            try:
                if printing:
                    request = args[0]
                    _request = DSRequest(request)
                    self.print(_request)
                    self.print(request.COOKIES, 'request.COOKIES')
                    self.print(request.GET, 'request.GET')
                    self.print(request.POST, 'request.POST')
                    self.print(request.META, 'request.META')

                res = self.func(*args, **kwargs)
                if printing_res:
                    self.print(res)
                return res
            except Exception as ex:
                exc_info = sys.exc_info()
                stackTrace = traceback.format_exception(*exc_info)
                message = str(ex)
                for x in stackTrace:
                    logger.error(x)

                del exc_info

                return JsonResponse(DSResponseFailure(message=message, stackTrace=stackTrace).response)

    return JE


def JsonWSPostResponseWithException(printing=False, printing_res=False):
    class JE:
        def __init__(self, func):
            self.func = func
            self.pp = pprint.PrettyPrinter(indent=4)

        def print(self, o, str=None):
            if str:
                print(f"=========== Begin {str} ========================")
            else:
                print(f"=========== Begin {o.__class__.__name__} ========================")
            if isinstance(o, dict):
                self.pp.pprint(RequestData(o).getDataWithOutField(['pp', '_state']).dict())
            elif isinstance(o, object):
                self.pp.pprint(RequestData(o.__dict__).getDataWithOutField(['pp', '_state']).dict())

            if str:
                print(f"=========== End   {str} ========================")
            else:
                print(f"=========== End   {o.__class__.__name__} ========================")
            print("\n")

        def __call__(self, *args, **kwargs):
            from isc_common.json import BytesToJson
            request = args[0]

            self.port = request.POST.get('ws_port')
            if self.port == None:
                self.port = settings.WS_PORT
                if not self.port:
                    raise Exception(Common.arraund_error('Do not have ws_port'))

            self.host = request.POST.get('ws_host')
            if self.host == None:
                self.host = settings.WS_HOST
                if not self.host:
                    raise Exception(Common.arraund_error('Do not have ws_host'))

            self.channel = request.POST.get('ws_channel')
            if self.channel == None:
                body = BytesToJson(request.body)
                body = Body(body, request.session)
                if settings.WS_CHANNEL != None:
                    self.channel = f'''{settings.WS_CHANNEL}_{body.login}'''
                    if self.channel == None:
                        raise Exception(Common.arraund_error('Do not have ws_channel'))

            try:
                if printing:
                    _request = DSRequest(request)
                    self.print(_request)
                    self.print(request.COOKIES, 'request.COOKIES')
                    self.print(request.GET, 'request.GET')
                    self.print(request.POST, 'request.POST')
                    self.print(request.META, 'request.META')

                res = self.func(*args, **kwargs)
                if printing_res:
                    self.print(res)

                return res
            except Exception as ex:
                exc_info = sys.exc_info()
                stackTrace = traceback.format_exception(*exc_info)
                message = str(ex)

                try:
                    ws = create_connection(f"ws://{self.host}:{self.port}/ws/{self.channel}/")
                    ws.send(json.dumps(dict(message=message, stackTrace=stackTrace, type="error")))
                    ws.close()
                except Exception as ex:

                    exc_info = sys.exc_info()
                    message = str(ex)
                    stackTrace = traceback.format_exception(*exc_info)

                    logging.error(message)

                    for x in stackTrace:
                        logger.error(x)

                    del exc_info

                for x in stackTrace:
                    logger.error(x)

                return JsonResponse(DSResponseOk().response)

    return JE


class DSResponse(RPCResponse):

    def normal_round(self, n):
        if n - math.floor(n) < 0.5:
            return math.floor(n)
        return math.ceil(n)

    def __init__(self,
                 request,
                 clientContext=None,
                 data=None,
                 message=None,
                 stackTrace=None,
                 httpHeaders=None,
                 httpResponseCode=None,
                 httpResponseText=None,
                 status=RPCResponseConstant.statusSuccess,
                 transactionNum=None,
                 dataSource=None,
                 errors=None,
                 invalidateCache=None,
                 operationId=None,
                 operationType=None,
                 queueStatus=None,
                 totalRows=None
                 ):
        RPCResponse.__init__(self,
                             clientContext=clientContext,
                             data=data,
                             httpHeaders=httpHeaders,
                             httpResponseCode=httpResponseCode,
                             httpResponseText=httpResponseText,
                             status=status,
                             transactionNum=transactionNum)

        request = DSRequest(request=request) if request else None
        self.dataSource = dataSource
        self.endRow = request.endRow if request else None
        self.startRow = request.startRow if request else None
        self.drawAheadRatio = request.drawAheadRatio if request else 1.2
        self.errors = errors
        self.invalidateCache = invalidateCache
        self.operationId = operationId
        self.operationType = operationType
        self.queueStatus = queueStatus

        self.message = message
        self.stackTrace = stackTrace
        self.status = status

        if isinstance(data, QuerySet):
            data = list(data)

        if isinstance(data, list):

            if self.startRow is not None and self.endRow is not None:
                qty = self.endRow - self.startRow
                len_data = len(self.data)

                if len_data == 0:
                    self.totalRows = 0
                else:
                    if len_data == qty:
                        self.totalRows = self.startRow + self.normal_round(qty * self.drawAheadRatio)
                    else:
                        self.totalRows = self.startRow + len_data

                # logger.debug(f'startRow: {self.startRow}, endRow: {self.endRow}, qty: {qty}, len_data: {len_data}, totalRows: {self.totalRows}')
            else:
                self.totalRows = totalRows
        else:
            self.totalRows = totalRows

    @property
    def response(self):
        setAttr(self.dict, 'startRow', self.startRow)
        setAttr(self.dict, 'endRow', self.endRow)
        setAttr(self.dict, 'totalRows', self.totalRows)
        # logger.debug(f'self.dict: {self.dict}')
        return dict(response=self.dict)


class DSResponseAdd(DSResponse):
    def __init__(self, data, status):
        if isinstance(data, Model):
            data = [model_to_dict(data)]

        DSResponse.__init__(self,
                            request=None,
                            clientContext=None,
                            data=data,
                            httpHeaders=None,
                            httpResponseCode=None,
                            httpResponseText=None,
                            status=status,
                            transactionNum=None,
                            dataSource=None,
                            errors=None,
                            invalidateCache=None,
                            operationId=None,
                            operationType=None,
                            queueStatus=None,
                            totalRows=None,
                            )


class DSResponseCopy(DSResponse):
    def __init__(self, data, status):
        if isinstance(data, Model):
            data = [model_to_dict(data)]

        DSResponse.__init__(self,
                            request=None,
                            clientContext=None,
                            data=data,
                            httpHeaders=None,
                            httpResponseCode=None,
                            httpResponseText=None,
                            status=status,
                            transactionNum=None,
                            dataSource=None,
                            errors=None,
                            invalidateCache=None,
                            operationId=None,
                            operationType=None,
                            queueStatus=None,
                            totalRows=None,
                            )


class DSResponseUpdate(DSResponse):
    def __init__(self, data, status):
        if isinstance(data, Model):
            data = [model_to_dict(data)]

        DSResponse.__init__(self,
                            request=None,
                            clientContext=None,
                            data=data,
                            httpHeaders=None,
                            httpResponseCode=None,
                            httpResponseText=None,
                            status=status,
                            transactionNum=None,
                            dataSource=None,
                            errors=None,
                            invalidateCache=None,
                            operationId=None,
                            operationType=None,
                            queueStatus=None,
                            totalRows=None,
                            )

        @property
        def response(self):
            return dict(response=dict(status=self.status, data=self.data))


class DSResponseLookup(DSResponse):
    def __init__(self, data, status):
        if isinstance(data, Model):
            data = [model_to_dict(data)]

        DSResponse.__init__(self,
                            request=None,
                            clientContext=None,
                            data=data,
                            httpHeaders=None,
                            httpResponseCode=None,
                            httpResponseText=None,
                            status=status,
                            transactionNum=None,
                            dataSource=None,
                            errors=None,
                            invalidateCache=None,
                            operationId=None,
                            operationType=None,
                            queueStatus=None,
                            totalRows=None,
                            )

        @property
        def response(self):
            return dict(response=dict(status=self.status, data=self.data))


class DSResponseOk(DSResponse):
    def __init__(self):
        DSResponse.__init__(self,
                            request=None,
                            clientContext=None,
                            data=[],
                            httpHeaders=None,
                            httpResponseCode=None,
                            httpResponseText=None,
                            status=RPCResponseConstant.statusSuccess,
                            transactionNum=None,
                            dataSource=None,
                            errors=None,
                            invalidateCache=None,
                            operationId=None,
                            operationType=None,
                            queueStatus=None,
                            totalRows=None,
                            )

    @property
    def response(self):
        return dict(response=dict(status=self.status))


class DSResponseFailure(DSResponse):
    def __init__(self,
                 message=None,
                 stackTrace=None
                 ):
        DSResponse.__init__(self,
                            request=None,
                            clientContext=None,
                            message=message,
                            stackTrace=stackTrace,
                            httpHeaders=None,
                            httpResponseCode=None,
                            httpResponseText=None,
                            status=RPCResponseConstant.statusFailure,
                            transactionNum=None,
                            dataSource=None,
                            errors=None,
                            invalidateCache=None,
                            operationId=None,
                            operationType=None,
                            queueStatus=None,
                            totalRows=None,
                            )

    @property
    def response(self):
        return dict(response=dict(
            status=-1,
            data=dict(
                error=dict(
                    message=self.message,
                    stackTrace=self.stackTrace
                )
            )
        ))



# Generated by Django 2.2.4 on 2019-08-06 06:44

from django.db import migrations
import isc_common.fields.description_field


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0034_messages_theme_access'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='messages_state',
            options={'verbose_name': 'Состояние задачи'},
        ),
        migrations.AlterModelOptions(
            name='messages_theme',
            options={'verbose_name': 'Темы задач'},
        ),
        migrations.AlterModelOptions(
            name='messages_theme_access',
            options={'verbose_name': 'Доступы к темам задач'},
        ),
        migrations.AlterModelOptions(
            name='messages_whom',
            options={'verbose_name': 'Получатель задач'},
        ),
        migrations.AlterField(
            model_name='messages',
            name='message',
            field=isc_common.fields.description_field.DescriptionField(verbose_name='Тело задачи'),
        ),
    ]



# Generated by Django 2.2.5 on 2019-09-05 17:01

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('twits', '0005_chat_user_user_common_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='chat_user_user',
            name='common_id',
            field=models.PositiveIntegerField(),
        ),
        migrations.AlterUniqueTogether(
            name='chat_user_user',
            unique_together={('user_sender', 'user_reciver', 'common_id')},
        ),
    ]



from isc_common.isc.data_binding.criterion import Criterion


class AdvancedCriteria(Criterion):
    _constructor = None

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            if isinstance(v, list):
                setattr(self, k, [AdvancedCriteria(**criterion) if criterion.get('_constructor') == 'AdvancedCriteria' else Criterion(**criterion) for criterion in v])
            else:
                setattr(self, k, v() if callable(v) else v)



import os
import shutil


def mekeDirs(name, logger, mode=0o777, exist_ok=False):
    try:
        os.makedirs(name=name, exist_ok=exist_ok, mode=mode)
        if not os.path.exists(name):
            logger.error(f'dirs: {name} not created.')
            return False
        return True
    except FileNotFoundError:
        logger.error(f'dirs: {name} not created.')
        return False


def copyTwo(src, dst, follow_symlinks=True):
    dir, _file = os.path.split(dst)
    os.chdir(dir)
    try:
        shutil.copy2(src=src, dst=_file, follow_symlinks=follow_symlinks)
        return True
    except FileNotFoundError:
        return False



# Generated by Django 2.2.5 on 2019-09-09 14:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0063_auto_20190909_1357'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='messages',
            options={'ordering': ('date_create',), 'verbose_name': 'Сообщения'},
        ),
    ]



from code_generator.app.model import Model


class AppMainWindow(Model):
    identifier = 'AppMainWindow_ID'
    loadSchemas = False
    name = 'Главное окно.'
    logoSrc = None
    logoHeight = 0
    useExtUserEditor = False
    addedEditors = None
    customUserMenu = None
    messageDS = None
    managedUsersGroups = None
    template_name = 'ts/common/app.pmd'
    output_file_name = 'mainWindow.ts'
    code_type = 'ts'
    appImageDir = None

    def write_entity(self):
        super().write_entity()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.class_name = self.__class__.__name__
        if self.appImageDir == None:
            self.appImageDir = f'/static/{self.image_dir_name}/images/'



import logging

from django.db.models import Model, TextField, Manager, PositiveIntegerField
from isc_common.fields.related import ForeignKeyCascade
from files_blk.models.files_group import Files_group
from isc_common.models.audit import AuditModel

logger = logging.getLogger(__name__)


class Files(AuditModel):
    path = TextField(db_index=True)
    file_size = PositiveIntegerField()
    group = ForeignKeyCascade(Files_group)

    objects = Manager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Файлы'
